@@ -34,6 +34,7 @@
 #endif
 
 #include <QFileDialog>
+#include <QFileSystemWatcher>
 #include <QMessageBox>
 #include <QTimer>
 #include <QDesktopServices>
@@ -69,6 +70,7 @@
 #include "qmacapplication.h"
 void qt_mac_set_dock_menu(QMenu *menu);
 #endif
+#include "lineedit.h"
 
 using namespace libtorrent;
 
@@ -83,7 +85,7 @@ using namespace libtorrent;
 // Constructor
 GUI::GUI(QWidget *parent, QStringList torrentCmdLine) : QMainWindow(parent), force_exit(false) {
   setupUi(this);
-
+  ui_locked = Preferences::isUILocked();
   setWindowTitle(tr("qBittorrent %1", "e.g: qBittorrent v0.x").arg(QString::fromUtf8(VERSION)));
   displaySpeedInTitle = Preferences::speedInTitleBar();
   // Setting icons
@@ -107,6 +109,11 @@ GUI::GUI(QWidget *parent, QStringList torrentCmdLine) : QMainWindow(parent), for
   actionSet_global_upload_limit->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/seeding.png")));
   actionSet_global_download_limit->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/download.png")));
   actionDocumentation->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/qb_question.png")));
+  actionLock_qBittorrent->setIcon(QIcon(QString::fromUtf8(":/Icons/oxygen/encrypted32.png")));
+  lockMenu = new QMenu();
+  QAction *defineUiLockPasswdAct = lockMenu->addAction(tr("Set the password..."));
+  connect(defineUiLockPasswdAct, SIGNAL(triggered()), this, SLOT(defineUILockPassword()));
+  actionLock_qBittorrent->setMenu(lockMenu);
   prioSeparator = toolBar->insertSeparator(actionDecreasePriority);
   prioSeparator2 = menu_Edit->insertSeparator(actionDecreasePriority);
   prioSeparator->setVisible(false);
@@ -149,6 +156,13 @@ GUI::GUI(QWidget *parent, QStringList torrentCmdLine) : QMainWindow(parent), for
   connect(transferList, SIGNAL(torrentStatusUpdate(uint,uint,uint,uint,uint)), this, SLOT(updateNbTorrents(uint,uint,uint,uint,uint)));
   vboxLayout->addWidget(tabs);
 
+  // Name filter
+  search_filter = new LineEdit();
+  QAction *separatorBFSearch = toolBar->insertSeparator(actionLock_qBittorrent);
+  toolBar->insertWidget(separatorBFSearch, search_filter);
+  search_filter->setFixedWidth(200);
+  connect(search_filter, SIGNAL(textChanged(QString)), transferList, SLOT(applyNameFilter(QString)));
+
   // Transfer list slots
   connect(actionStart, SIGNAL(triggered()), transferList, SLOT(startSelectedTorrents()));
   connect(actionStart_All, SIGNAL(triggered()), transferList, SLOT(startAllTorrents()));
@@ -160,10 +174,6 @@ GUI::GUI(QWidget *parent, QStringList torrentCmdLine) : QMainWindow(parent), for
 
   // Configure BT session according to options
   loadPreferences(false);
-  // Resume unfinished torrents
-  BTSession->startUpTorrents();
-  // Add torrent given on command line
-  processParams(torrentCmdLine);
 
   // Start connection checking timer
   guiUpdater = new QTimer(this);
@@ -187,6 +197,7 @@ GUI::GUI(QWidget *parent, QStringList torrentCmdLine) : QMainWindow(parent), for
   actionSearch_engine->setChecked(Preferences::isSearchEnabled());
   displaySearchTab(actionSearch_engine->isChecked());
   displayRSSTab(actionRSS_Reader->isChecked());
+  actionShutdown_when_downloads_complete->setChecked(Preferences::shutdownWhenDownloadsComplete());
 
   show();
 
@@ -203,10 +214,23 @@ GUI::GUI(QWidget *parent, QStringList torrentCmdLine) : QMainWindow(parent), for
   }while(transferListFilters->getStatusFilters()->verticalScrollBar()->sliderPosition() > 0);
   transferListFilters->getStatusFilters()->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
-  if(Preferences::startMinimized()) {
-    showMinimized();
+  if(ui_locked) {
+    hide();
+  } else {
+    if(Preferences::startMinimized())
+      showMinimized();
   }
 
+  // Start watching the executable for updates
+  executable_watcher = new QFileSystemWatcher();
+  connect(executable_watcher, SIGNAL(fileChanged(QString)), this, SLOT(notifyOfUpdate(QString)));
+  executable_watcher->addPath(qApp->applicationFilePath());
+
+  // Resume unfinished torrents
+  BTSession->startUpTorrents();
+  // Add torrent given on command line
+  processParams(torrentCmdLine);
+
   qDebug("GUI Built");
 #ifdef Q_WS_WIN
   if(!Preferences::neverCheckFileAssoc() && !Preferences::isFileAssocOk()) {
@@ -239,9 +263,13 @@ GUI::~GUI() {
   properties->saveSettings();
   disconnect(tabs, SIGNAL(currentChanged(int)), this, SLOT(tab_changed(int)));
   // Delete other GUI objects
+  if(executable_watcher)
+    delete executable_watcher;
   delete status_bar;
+  delete search_filter;
   delete transferList;
   delete guiUpdater;
+  delete lockMenu;
   if(createTorrentDlg)
     delete createTorrentDlg;
   if(console)
@@ -276,10 +304,8 @@ GUI::~GUI() {
   delete switchTransferShortcut;
   delete switchRSSShortcut;
   // Delete BTSession objects
+  qDebug("Deleting BTSession");
   delete BTSession;
-  // Deleting remaining top level widgets
-  qDebug("Deleting remaining top level widgets");
-
   // May freeze for a few seconds after the next line
   // because the Bittorrent session proxy will
   // actually be deleted now and destruction
@@ -287,6 +313,35 @@ GUI::~GUI() {
   qDebug("Exiting GUI destructor...");
 }
 
+void GUI::defineUILockPassword() {
+  QString old_pass_md5 = Preferences::getUILockPasswordMD5();
+  if(old_pass_md5.isNull()) old_pass_md5 = "";
+  bool ok = false;
+  QString new_clear_password = QInputDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, old_pass_md5, &ok);
+  if(ok) {
+    if(new_clear_password != old_pass_md5) {
+      Preferences::setUILockPassword(new_clear_password);
+    }
+    QMessageBox::information(this, tr("Password update"), tr("The UI lock password has been successfully updated"));
+  }
+}
+
+void GUI::on_actionLock_qBittorrent_triggered() {
+  // Check if there is a password
+  if(Preferences::getUILockPasswordMD5().isEmpty()) {
+    // Ask for a password
+    bool ok = false;
+    QString clear_password = QInputDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, "", &ok);
+    if(!ok) return;
+    Preferences::setUILockPassword(clear_password);
+  }
+  // Lock the interface
+  ui_locked = true;
+  Preferences::setUILocked(true);
+  myTrayIconMenu->setEnabled(false);
+  hide();
+}
+
 void GUI::displayRSSTab(bool enable) {
   if(enable) {
     // RSS tab
@@ -528,10 +583,41 @@ void GUI::setTabText(int index, QString text) const {
   tabs->setTabText(index, text);
 }
 
+bool GUI::unlockUI() {
+  bool ok = false;
+  QString clear_password = QInputDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, "", &ok);
+  if(!ok) return false;
+  QString real_pass_md5 = Preferences::getUILockPasswordMD5();
+  QCryptographicHash md5(QCryptographicHash::Md5);
+  md5.addData(clear_password.toLocal8Bit());
+  QString password_md5 = md5.result().toHex();
+  if(real_pass_md5 == password_md5) {
+    ui_locked = false;
+    Preferences::setUILocked(false);
+    myTrayIconMenu->setEnabled(true);
+    return true;
+  }
+  QMessageBox::warning(this, tr("Invalid password"), tr("The password is invalid"));
+  return false;
+}
+
+void GUI::notifyOfUpdate(QString) {
+  // Show restart message
+  status_bar->showRestartRequired();
+  // Delete the executable watcher
+  delete executable_watcher;
+  executable_watcher = 0;
+}
+
 // Toggle Main window visibility
 void GUI::toggleVisibility(QSystemTrayIcon::ActivationReason e) {
   if(e == QSystemTrayIcon::Trigger || e == QSystemTrayIcon::DoubleClick) {
     if(isHidden()) {
+      if(ui_locked) {
+        // Ask for UI lock password
+        if(!unlockUI())
+          return;
+      }
       show();
       if(isMinimized()) {
         if(isMaximized()) {
@@ -738,8 +824,8 @@ void GUI::on_actionOpen_triggered() {
   // Open File Open Dialog
   // Note: it is possible to select more than one file
   const QStringList pathsList = QFileDialog::getOpenFileNames(0,
-                                                               tr("Open Torrent Files"), settings.value(QString::fromUtf8("MainWindowLastDir"), QDir::homePath()).toString(),
-                                                               tr("Torrent Files")+QString::fromUtf8(" (*.torrent)"));
+                                                              tr("Open Torrent Files"), settings.value(QString::fromUtf8("MainWindowLastDir"), QDir::homePath()).toString(),
+                                                              tr("Torrent Files")+QString::fromUtf8(" (*.torrent)"));
   if(!pathsList.empty()) {
     const bool useTorrentAdditionDialog = settings.value(QString::fromUtf8("Preferences/Downloads/AdditionDialog"), true).toBool();
     const uint listSize = pathsList.size();
@@ -821,6 +907,7 @@ void GUI::loadPreferences(bool configure_session) {
   BTSession->addConsoleMessage(tr("Options were saved successfully."));
 #ifndef Q_WS_MAC
   const bool newSystrayIntegration = Preferences::systrayIntegration();
+  actionLock_qBittorrent->setEnabled(newSystrayIntegration);
   if(newSystrayIntegration != (systrayIcon!=0)) {
     if(newSystrayIntegration) {
       // create the trayicon
@@ -849,6 +936,8 @@ void GUI::loadPreferences(bool configure_session) {
     toolBar->setVisible(true);
     toolBar->layout()->setSpacing(7);
   } else {
+    // Clear search filter before hiding the top toolbar
+    search_filter->clear();
     toolBar->setVisible(false);
   }
   const uint new_refreshInterval = Preferences::getRefreshInterval();
@@ -1031,6 +1120,8 @@ QMenu* GUI::getTrayIconMenu() {
   myTrayIconMenu->addAction(actionPause_All);
   myTrayIconMenu->addSeparator();
   myTrayIconMenu->addAction(actionExit);
+  if(ui_locked)
+    myTrayIconMenu->setEnabled(false);
   return myTrayIconMenu;
 }
 
@@ -1066,6 +1157,11 @@ void GUI::on_actionTop_tool_bar_triggered() {
   Preferences::setToolbarDisplayed(is_visible);
 }
 
+void GUI::on_actionShutdown_when_downloads_complete_triggered() {
+  bool is_checked = static_cast<QAction*>(sender())->isChecked();
+  Preferences::setShutdownWhenDownloadsComplete(is_checked);
+}
+
 void GUI::on_actionSpeed_in_title_bar_triggered() {
   displaySpeedInTitle = static_cast<QAction*>(sender())->isChecked();
   Preferences::showSpeedInTitleBar(displaySpeedInTitle);
