@@ -57,6 +57,8 @@ class about;
 class createtorrent;
 class downloadFromURL;
 class HidableTabWidget;
+class LineEdit;
+class QFileSystemWatcher;
 
 class GUI : public QMainWindow, private Ui::MainWindow{
   Q_OBJECT
@@ -69,6 +71,7 @@ public:
   QWidget* getCurrentTabWidget() const;
   TransferListWidget* getTransferList() const { return transferList; }
   QMenu* getTrayIconMenu();
+  PropertiesWidget *getProperties() const { return properties; }
 
 public slots:
   void trackerAuthenticationRequired(QTorrentHandle& h);
@@ -97,6 +100,10 @@ protected slots:
   void handleDownloadFromUrlFailure(QString, QString) const;
   void createSystrayDelayed();
   void tab_changed(int);
+  void on_actionLock_qBittorrent_triggered();
+  void defineUILockPassword();
+  bool unlockUI();
+  void notifyOfUpdate(QString);
   // Keyboard shortcuts
   void createKeyboardShortcuts();
   void displayTransferTab() const;
@@ -130,6 +137,7 @@ protected:
   void displaySearchTab(bool enable);
 
 private:
+  QFileSystemWatcher *executable_watcher;
   // Bittorrent
   Bittorrent *BTSession;
   QList<QPair<QTorrentHandle,QString> > unauthenticated_trackers; // Still needed?
@@ -150,6 +158,8 @@ private:
   PropertiesWidget *properties;
   bool displaySpeedInTitle;
   bool force_exit;
+  bool ui_locked;
+  LineEdit *search_filter;
   // Keyboard shortcuts
   QShortcut *switchSearchShortcut;
   QShortcut *switchSearchShortcut2;
@@ -160,6 +170,7 @@ private:
   QAction *prioSeparator2;
   QSplitter *hSplitter;
   QSplitter *vSplitter;
+  QMenu *lockMenu;
   // Search
   QPointer<SearchEngine> searchEngine;
   // RSS
@@ -170,6 +181,7 @@ private slots:
     void on_actionRSS_Reader_triggered();
     void on_actionSpeed_in_title_bar_triggered();
     void on_actionTop_tool_bar_triggered();
+    void on_actionShutdown_when_downloads_complete_triggered();
 };
 
 #endif
