@@ -5,19 +5,22 @@
 #include <QHeaderView>
 #include <QSpinBox>
 #include <QCheckBox>
+#include <QComboBox>
+#include <QNetworkInterface>
 #include <libtorrent/version.hpp>
 #include "preferences.h"
 
 enum AdvSettingsCols {PROPERTY, VALUE};
-enum AdvSettingsRows {DISK_CACHE, OUTGOING_PORT_MIN, OUTGOING_PORT_MAX, IGNORE_LIMIT_LAN, COUNT_OVERHEAD, RECHECK_COMPLETED, LIST_REFRESH, RESOLVE_COUNTRIES, RESOLVE_HOSTS };
-#define ROW_COUNT 9
+enum AdvSettingsRows {DISK_CACHE, OUTGOING_PORT_MIN, OUTGOING_PORT_MAX, IGNORE_LIMIT_LAN, COUNT_OVERHEAD, RECHECK_COMPLETED, LIST_REFRESH, RESOLVE_COUNTRIES, RESOLVE_HOSTS, MAX_HALF_OPEN, SUPER_SEEDING, NETWORK_IFACE, PROGRAM_NOTIFICATIONS };
+#define ROW_COUNT 13
 
 class AdvancedSettings: public QTableWidget {
   Q_OBJECT
 
 private:
-  QSpinBox *spin_cache, *outgoing_ports_min, *outgoing_ports_max, *spin_list_refresh;
-  QCheckBox *cb_ignore_limits_lan, *cb_count_overhead, *cb_recheck_completed, *cb_resolve_countries, *cb_resolve_hosts;
+  QSpinBox *spin_cache, *outgoing_ports_min, *outgoing_ports_max, *spin_list_refresh, *spin_maxhalfopen;
+  QCheckBox *cb_ignore_limits_lan, *cb_count_overhead, *cb_recheck_completed, *cb_resolve_countries, *cb_resolve_hosts, *cb_super_seeding, *cb_program_notifications;
+  QComboBox *combo_iface;
 
 public:
   AdvancedSettings(QWidget *parent=0): QTableWidget(parent) {
@@ -46,6 +49,10 @@ public:
     delete spin_list_refresh;
     delete cb_resolve_countries;
     delete cb_resolve_hosts;
+    delete spin_maxhalfopen;
+    delete cb_super_seeding;
+    delete combo_iface;
+    delete cb_program_notifications;
   }
 
 public slots:
@@ -66,6 +73,21 @@ public slots:
     // Peer resolution
     Preferences::resolvePeerCountries(cb_resolve_countries->isChecked());
     Preferences::resolvePeerHostNames(cb_resolve_hosts->isChecked());
+    // Max Half-Open connections
+    Preferences::setMaxHalfOpenConnections(spin_maxhalfopen->value());
+#if LIBTORRENT_VERSION_MINOR > 14
+    // Super seeding
+    Preferences::enableSuperSeeding(cb_super_seeding->isChecked());
+#endif
+    // Network interface
+    if(combo_iface->currentIndex() == 0) {
+      // All interfaces (default)
+      Preferences::setNetworkInterface(QString::null);
+    } else {
+      Preferences::setNetworkInterface(combo_iface->currentText());
+    }
+    // Program notification
+    Preferences::useProgramNotification(cb_program_notifications->isChecked());
   }
 
 protected slots:
@@ -134,6 +156,45 @@ protected slots:
     connect(cb_resolve_hosts, SIGNAL(toggled(bool)), this, SLOT(emitSettingsChanged()));
     cb_resolve_hosts->setChecked(Preferences::resolvePeerHostNames());
     setCellWidget(RESOLVE_HOSTS, VALUE, cb_resolve_hosts);
+    // Max Half Open connections
+    setItem(MAX_HALF_OPEN, PROPERTY, new QTableWidgetItem(tr("Maximum number of half-open connections [0: Disabled]")));
+    spin_maxhalfopen = new QSpinBox();
+    connect(spin_maxhalfopen, SIGNAL(valueChanged(int)), this, SLOT(emitSettingsChanged()));
+    spin_maxhalfopen->setMinimum(0);
+    spin_maxhalfopen->setMaximum(99999);
+    spin_maxhalfopen->setValue(Preferences::getMaxHalfOpenConnections());
+    setCellWidget(MAX_HALF_OPEN, VALUE, spin_maxhalfopen);
+    // Super seeding
+    setItem(SUPER_SEEDING, PROPERTY, new QTableWidgetItem(tr("Strict super seeding")));
+    cb_super_seeding = new QCheckBox();
+    connect(cb_super_seeding, SIGNAL(toggled(bool)), this, SLOT(emitSettingsChanged()));
+#if LIBTORRENT_VERSION_MINOR > 14
+    cb_super_seeding->setChecked(Preferences::isSuperSeedingEnabled());
+#else
+    cb_super_seeding->setEnabled(false);
+#endif
+    setCellWidget(SUPER_SEEDING, VALUE, cb_super_seeding);
+    // Network interface
+    setItem(NETWORK_IFACE, PROPERTY, new QTableWidgetItem(tr("Network Interface (requires restart)")));
+    combo_iface = new QComboBox;
+    combo_iface->addItem(tr("Any interface", "i.e. Any network interface"));
+    const QString current_iface = Preferences::getNetworkInterface();
+    int i = 1;
+    foreach(const QNetworkInterface& iface, QNetworkInterface::allInterfaces()) {
+      if(iface.name() == "lo") continue;
+      combo_iface->addItem(iface.name());
+      if(!current_iface.isEmpty() && iface.name() == current_iface)
+        combo_iface->setCurrentIndex(i);
+      ++i;
+    }
+    connect(combo_iface, SIGNAL(currentIndexChanged(int)), this, SLOT(emitSettingsChanged()));
+    setCellWidget(NETWORK_IFACE, VALUE, combo_iface);
+    // Program notifications
+    setItem(PROGRAM_NOTIFICATIONS, PROPERTY, new QTableWidgetItem(tr("Display program notification baloons")));
+    cb_program_notifications = new QCheckBox();
+    connect(cb_program_notifications, SIGNAL(toggled(bool)), this, SLOT(emitSettingsChanged()));
+    cb_program_notifications->setChecked(Preferences::useProgramNotification());
+    setCellWidget(PROGRAM_NOTIFICATIONS, VALUE, cb_program_notifications);
   }
 
   void emitSettingsChanged() {
