@@ -38,6 +38,7 @@
 //#include "proplistdelegate.h"
 #include "torrentpersistentdata.h"
 #include <QDebug>
+#include <QTranslator>
 
 EventManager::EventManager(QObject *parent, Bittorrent *BTSession)
   : QObject(parent), BTSession(BTSession)
@@ -124,8 +125,20 @@ QList<QVariantMap> EventManager::getPropFilesInfo(QString hash) const {
 
 void EventManager::setGlobalPreferences(QVariantMap m) const {
   // UI
-  if(m.contains("locale"))
-    Preferences::setLocale(m["locale"].toString());
+  if(m.contains("locale")) {
+    QString locale = m["locale"].toString();
+    if(Preferences::getLocale() != locale) {
+      QTranslator *translator = new QTranslator;
+      if(translator->load(QString::fromUtf8(":/lang/qbittorrent_") + locale)){
+        qDebug("%s locale recognized, using translation.", qPrintable(locale));
+      }else{
+        qDebug("%s locale unrecognized, using default (en_GB).", qPrintable(locale));
+      }
+      qApp->installTranslator(translator);
+    }
+
+    Preferences::setLocale(locale);
+  }
   // Downloads
   if(m.contains("save_path"))
     Preferences::setSavePath(m["save_path"].toString());
@@ -163,6 +176,16 @@ void EventManager::setGlobalPreferences(QVariantMap m) const {
   }
   if(m.contains("export_dir"))
     Preferences::setExportDir(m["export_dir"].toString());
+  if(m.contains("mail_notification_enabled"))
+    Preferences::setMailNotificationEnabled(m["mail_notification_enabled"].toBool());
+  if(m.contains("mail_notification_email"))
+    Preferences::setMailNotificationEmail(m["mail_notification_email"].toString());
+  if(m.contains("mail_notification_smtp"))
+    Preferences::setMailNotificationSMTP(m["mail_notification_smtp"].toString());
+  if(m.contains("autorun_enabled"))
+    Preferences::setAutoRunEnabled(m["autorun_enabled"].toBool());
+  if(m.contains("autorun_program"))
+    Preferences::setAutoRunProgram(m["autorun_program"].toString());
   if(m.contains("preallocate_all"))
     Preferences::preAllocateAllFiles(m["preallocate_all"].toBool());
   if(m.contains("queueing_enabled"))
@@ -265,6 +288,11 @@ QVariantMap EventManager::getGlobalPreferences() const {
   data["download_in_scan_dirs"] = var_list;
   data["export_dir_enabled"] = Preferences::isTorrentExportEnabled();
   data["export_dir"] = Preferences::getExportDir();
+  data["mail_notification_enabled"] = Preferences::isMailNotificationEnabled();
+  data["mail_notification_email"] = Preferences::getMailNotificationEmail();
+  data["mail_notification_smtp"] = Preferences::getMailNotificationSMTP();
+  data["autorun_enabled"] = Preferences::isAutoRunEnabled();
+  data["autorun_program"] = Preferences::getAutoRunProgram();
   data["preallocate_all"] = Preferences::preAllocateAllFiles();
   data["queueing_enabled"] = Preferences::isQueueingSystemEnabled();
   data["max_active_downloads"] = Preferences::getMaxActiveDownloads();
