@@ -109,7 +109,7 @@ QString HttpConnection::translateDocument(QString data) {
   bool found = false;
   do {
     found = false;
-    QRegExp regex(QString::fromUtf8("_\\(([\\w\\s?!:\\/\\(\\),µ&\\-\\.]+)\\)"));
+    QRegExp regex(QString::fromUtf8("_\\(([\\w\\s?!:\\/\\(\\),%µ&\\-\\.]+)\\)"));
     i = regex.indexIn(data, i);
     if(i >= 0) {
       //qDebug("Found translatable string: %s", regex.cap(1).toUtf8().data());
@@ -161,7 +161,7 @@ void HttpConnection::respond() {
     write();
     return;
   }
-  // Client sucessfuly authenticated, reset number of failed attempts
+  // Client successfully authenticated, reset number of failed attempts
   parent->resetNbFailedAttemptsForIp(peer_ip);
   QString url  = parser.url();
   // Favicon
@@ -500,6 +500,16 @@ void HttpConnection::respondCommand(QString command)
     if(h.is_valid()) h.queue_position_down();
     return;
   }
+  if(command == "topPrio") {
+    QTorrentHandle h = BTSession->getTorrentHandle(parser.post("hash"));
+    if(h.is_valid()) h.queue_position_top();
+    return;
+  }
+  if(command == "bottomPrio") {
+    QTorrentHandle h = BTSession->getTorrentHandle(parser.post("hash"));
+    if(h.is_valid()) h.queue_position_bottom();
+    return;
+  }
   if(command == "recheck"){
     recheckTorrent(parser.post("hash"));
     return;
