@@ -54,6 +54,7 @@ const int UNLEN = 256;
 
 #ifdef Q_WS_MAC
 #include <CoreServices/CoreServices.h>
+#include <Carbon/Carbon.h>
 #endif
 
 #ifndef Q_WS_WIN
@@ -67,6 +68,11 @@ const int UNLEN = 256;
 #include <winbase.h>
 #endif
 
+#ifdef Q_WS_X11
+#include <QDBusInterface>
+#include <QDBusMessage>
+#endif
+
 QString misc::QDesktopServicesDataLocation() {
 #ifdef Q_WS_WIN
   LPWSTR path=new WCHAR[256];
@@ -119,7 +125,7 @@ QString misc::QDesktopServicesCacheLocation() {
   QString path;
   QByteArray ba(2048, 0);
   if (FSRefMakePath(&ref, reinterpret_cast<UInt8 *>(ba.data()), ba.size()) == noErr)
-        path = QString::fromUtf8(ba).normalized(QString::NormalizationForm_C);
+    path = QString::fromUtf8(ba).normalized(QString::NormalizationForm_C);
   path += QLatin1Char('/') + qApp->applicationName();
   return path;
 #else
@@ -184,6 +190,81 @@ long long misc::freeDiskSpaceOnPath(QString path) {
 #endif
 }
 
+void misc::shutdownComputer() {
+#ifdef Q_WS_X11
+  // Use dbus to power off the system
+  // dbus-send --print-reply --system --dest=org.freedesktop.Hal /org/freedesktop/Hal/devices/computer org.freedesktop.Hal.Device.SystemPowerManagement.Shutdown
+  QDBusInterface computer("org.freedesktop.Hal", "/org/freedesktop/Hal/devices/computer", "org.freedesktop.Hal.Device.SystemPowerManagement", QDBusConnection::systemBus());
+  computer.call("Shutdown");
+#endif
+#ifdef Q_WS_MAC
+  AEEventID EventToSend = kAEShutDown;
+  AEAddressDesc targetDesc;
+  static const ProcessSerialNumber kPSNOfSystemProcess = { 0, kSystemProcess };
+  AppleEvent eventReply = {typeNull, NULL};
+  AppleEvent appleEventToSend = {typeNull, NULL};
+
+  OSStatus error = noErr;
+
+  error = AECreateDesc(typeProcessSerialNumber, &kPSNOfSystemProcess,
+                       sizeof(kPSNOfSystemProcess), &targetDesc);
+
+  if (error != noErr)
+  {
+    return;
+  }
+
+  error = AECreateAppleEvent(kCoreEventClass, EventToSend, &targetDesc,
+                             kAutoGenerateReturnID, kAnyTransactionID, &appleEventToSend);
+
+  AEDisposeDesc(&targetDesc);
+  if (error != noErr)
+  {
+      return;
+  }
+
+  error = AESend(&appleEventToSend, &eventReply, kAENoReply,
+                 kAENormalPriority, kAEDefaultTimeout, NULL, NULL);
+
+  AEDisposeDesc(&appleEventToSend);
+  if (error != noErr)
+  {
+    return;
+  }
+
+  AEDisposeDesc(&eventReply);
+#endif
+#ifdef Q_WS_WIN
+  HANDLE hToken;              // handle to process token
+  TOKEN_PRIVILEGES tkp;       // pointer to token structure
+  if(!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken))
+    return;
+  // Get the LUID for shutdown privilege.
+  LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME,
+                       &tkp.Privileges[0].Luid);
+
+  tkp.PrivilegeCount = 1;  // one privilege to set
+  tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
+
+  // Get shutdown privilege for this process.
+
+  AdjustTokenPrivileges(hToken, FALSE, &tkp, 0,
+                        (PTOKEN_PRIVILEGES) NULL, 0);
+
+  // Cannot test the return value of AdjustTokenPrivileges.
+
+  if (GetLastError() != ERROR_SUCCESS)
+    return;
+  bool ret = InitiateSystemShutdownA(0, tr("qBittorrent will shutdown the computer now because all downloads are complete.").toLocal8Bit().data(), 10, true, false);
+  qDebug("ret: %d", (int)ret);
+
+  // Disable shutdown privilege.
+  tkp.Privileges[0].Attributes = 0;
+  AdjustTokenPrivileges(hToken, FALSE, &tkp, 0,
+                        (PTOKEN_PRIVILEGES) NULL, 0);
+#endif
+}
+
 QString misc::truncateRootFolder(boost::intrusive_ptr<torrent_info> t) {
   if(t->num_files() == 1) {
     // Single file torrent
@@ -376,7 +457,7 @@ QPoint misc::screenCenter(QWidget *win) {
 
 QString misc::searchEngineLocation() {
   const QString location = QDir::cleanPath(QDesktopServicesDataLocation()
-                                            + QDir::separator() + "search_engine");
+                                           + QDir::separator() + "search_engine");
   QDir locationDir(location);
   if(!locationDir.exists())
     locationDir.mkpath(locationDir.absolutePath());
@@ -385,7 +466,7 @@ QString misc::searchEngineLocation() {
 
 QString misc::BTBackupLocation() {
   const QString location = QDir::cleanPath(QDesktopServicesDataLocation()
-                                            + QDir::separator() + "BT_backup");
+                                           + QDir::separator() + "BT_backup");
   QDir locationDir(location);
   if(!locationDir.exists())
     locationDir.mkpath(locationDir.absolutePath());
@@ -421,6 +502,7 @@ bool misc::isPreviewable(QString extension){
   if(extension == "AVI") return true;
   if(extension == "MP3") return true;
   if(extension == "OGG") return true;
+  if(extension == "OGV") return true;
   if(extension == "OGM") return true;
   if(extension == "WMV") return true;
   if(extension == "WMA") return true;
@@ -625,11 +707,11 @@ QList<int> misc::intListfromStringList(const QStringList &l) {
 
 QList<bool> misc::boolListfromStringList(const QStringList &l) {
   QList<bool> ret;
-    foreach(const QString &s, l) {
-      if(s == "1")
-        ret << true;
-      else
-        ret << false;
-    }
-    return ret;
+  foreach(const QString &s, l) {
+    if(s == "1")
+      ret << true;
+    else
+      ret << false;
+  }
+  return ret;
 }
