@@ -33,6 +33,7 @@
 
 #include <sstream>
 #include <QString>
+#include <QStringList>
 #include <QThread>
 #include <ctime>
 #include <boost/date_time/posix_time/posix_time_types.hpp>
@@ -74,6 +75,14 @@ public:
     return QString(o.str().c_str());
   }
 
+  static inline QString removeLastPathPart(QString path) {
+    if(path.isEmpty()) return path;
+    path = path.replace("\\", "/");
+    QStringList tmp = path.split("/");
+    tmp.removeLast();
+    return tmp.join("/");
+  }
+
   static inline sha1_hash QStringToSha1(const QString& s) {
     std::string str(s.toLocal8Bit().data());
     std::istringstream i(str);
@@ -82,6 +91,8 @@ public:
     return x;
   }
 
+  static void shutdownComputer();
+
   static bool safeRemove(QString file_path) {
     QFile MyFile(file_path);
     if(!MyFile.exists()) return true;
