@@ -37,6 +37,7 @@
 #include <QCloseEvent>
 #include <QDesktopWidget>
 #include <QStyleFactory>
+#include <QTranslator>
 
 #include <libtorrent/version.hpp>
 #include <time.h>
@@ -187,6 +188,7 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
     connect(checkShowSplash, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
     // Downloads tab
     connect(textSavePath, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
+    connect(textTempPath, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
     connect(checkAppendLabel, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
     connect(checkAppendqB, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
     connect(checkPreallocateAll, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
@@ -199,6 +201,11 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
     connect(checkTempFolder, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
     connect(addScanFolderButton, SIGNAL(clicked()), this, SLOT(enableApplyButton()));
     connect(removeScanFolderButton, SIGNAL(clicked()), this, SLOT(enableApplyButton()));
+    connect(groupMailNotification, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
+    connect(dest_email_txt, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
+    connect(smtp_server_txt, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
+    connect(autoRunBox, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
+    connect(autoRun_txt, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
     // Connection tab
     connect(spinPort, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
     connect(checkUPnP, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
@@ -347,10 +354,22 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
     QIniSettings settings("qBittorrent", "qBittorrent");
     // Apply style
     useStyle();
+    // Load the translation
+    QString locale = getLocale();
+    if(Preferences::getLocale() != locale) {
+      QTranslator *translator = new QTranslator;
+      if(translator->load(QString::fromUtf8(":/lang/qbittorrent_") + locale)){
+        qDebug("%s locale recognized, using translation.", qPrintable(locale));
+      }else{
+        qDebug("%s locale unrecognized, using default (en_GB).", qPrintable(locale));
+      }
+      qApp->installTranslator(translator);
+    }
+
     settings.beginGroup("Preferences");
     // General preferences
     settings.beginGroup("General");
-    settings.setValue(QString::fromUtf8("Locale"), getLocale());
+    settings.setValue(QString::fromUtf8("Locale"), locale);
     settings.setValue(QString::fromUtf8("Style"), getStyle());
     settings.setValue(QString::fromUtf8("AlternatingRowColors"), checkAltRowColors->isChecked());
     settings.setValue(QString::fromUtf8("SystrayEnabled"), systrayIntegration());
@@ -387,6 +406,11 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
     export_dir = export_dir.replace("\\", "/");
 #endif
     Preferences::setExportDir(export_dir);
+    Preferences::setMailNotificationEnabled(groupMailNotification->isChecked());
+    Preferences::setMailNotificationEmail(dest_email_txt->text());
+    Preferences::setMailNotificationSMTP(smtp_server_txt->text());
+    Preferences::setAutoRunEnabled(autoRunBox->isChecked());
+    Preferences::setAutoRunProgram(autoRun_txt->text());
     settings.setValue(QString::fromUtf8("DblClOnTorDl"), getActionOnDblClOnTorrentDl());
     settings.setValue(QString::fromUtf8("DblClOnTorFn"), getActionOnDblClOnTorrentFn());
     // End Downloads preferences
@@ -602,7 +626,11 @@ options_imp::options_imp(QWidget *parent):QDialog(parent){
 #endif
       textExportDir->setText(strValue);
     }
-
+    groupMailNotification->setChecked(Preferences::isMailNotificationEnabled());
+    dest_email_txt->setText(Preferences::getMailNotificationEmail());
+    smtp_server_txt->setText(Preferences::getMailNotificationSMTP());
+    autoRunBox->setChecked(Preferences::isAutoRunEnabled());
+    autoRun_txt->setText(Preferences::getAutoRunProgram());
     intValue = Preferences::getActionOnDblClOnTorrentDl();
     if(intValue >= actionTorrentDlOnDblClBox->count())
       intValue = 0;
