@@ -276,6 +276,36 @@ public:
     settings.setValue(QString::fromUtf8("Preferences/Downloads/TorrentExport"), path);
   }
 
+  static bool isMailNotificationEnabled() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value(QString::fromUtf8("Preferences/MailNotification/enabled"), false).toBool();
+  }
+
+  static void setMailNotificationEnabled(bool enabled) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    settings.setValue(QString::fromUtf8("Preferences/MailNotification/enabled"), enabled);
+  }
+
+  static QString getMailNotificationEmail() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value(QString::fromUtf8("Preferences/MailNotification/email"), "").toString();
+  }
+
+  static void setMailNotificationEmail(QString mail) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    settings.setValue(QString::fromUtf8("Preferences/MailNotification/email"), mail);
+  }
+
+  static QString getMailNotificationSMTP() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value(QString::fromUtf8("Preferences/MailNotification/smtp_server"), "smtp.changeme.com").toString();
+  }
+
+  static void setMailNotificationSMTP(QString smtp_server) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    settings.setValue(QString::fromUtf8("Preferences/MailNotification/smtp_server"), smtp_server);
+  }
+
   static int getActionOnDblClOnTorrentDl() {
     QIniSettings settings("qBittorrent", "qBittorrent");
     return settings.value(QString::fromUtf8("Preferences/Downloads/DblClOnTorDl"), 0).toInt();
@@ -851,6 +881,60 @@ public:
   }
 
   // Advanced settings
+
+  static void setUILockPassword(QString clear_password) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    QCryptographicHash md5(QCryptographicHash::Md5);
+    md5.addData(clear_password.toLocal8Bit());
+    QString md5_password = md5.result().toHex();
+    settings.setValue("Locking/password", md5_password);
+  }
+
+  static QString getUILockPasswordMD5() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value("Locking/password", QString()).toString();
+  }
+
+  static bool isUILocked() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value("Locking/locked", false).toBool();
+  }
+
+  static void setUILocked(bool locked) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.setValue("Locking/locked", locked);
+  }
+
+  static bool isAutoRunEnabled() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value("AutoRun/enabled", false).toBool();
+  }
+
+  static void setAutoRunEnabled(bool enabled) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.setValue("AutoRun/enabled", enabled);
+  }
+
+  static void setAutoRunProgram(QString program) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    settings.setValue("AutoRun/program", program);
+  }
+
+  static QString getAutoRunProgram() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value("AutoRun/program", QString()).toString();
+  }
+
+  static bool shutdownWhenDownloadsComplete() {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    return settings.value(QString::fromUtf8("Preferences/Downloads/AutoShutDownOnCompletion"), false).toBool();
+  }
+
+  static void setShutdownWhenDownloadsComplete(bool shutdown) {
+    QIniSettings settings("qBittorrent", "qBittorrent");
+    settings.setValue(QString::fromUtf8("Preferences/Downloads/AutoShutDownOnCompletion"), shutdown);
+  }
+
   static uint diskCacheSize() {
     QIniSettings settings("qBittorrent", "qBittorrent");
     return settings.value(QString::fromUtf8("Preferences/Downloads/DiskCache"), 16).toUInt();
