@@ -227,7 +227,7 @@ int QTorrentHandle::num_incomplete() const {
 
 QString QTorrentHandle::save_path() const {
   Q_ASSERT(h.is_valid());
-  return misc::toQString(h.save_path().string()).replace("\\", "/");
+  return misc::toQStringU(h.save_path().string()).replace("\\", "/");
 }
 
 #if LIBTORRENT_VERSION_MINOR > 14
@@ -387,6 +387,14 @@ QStringList QTorrentHandle::files_path() const {
   return res;
 }
 
+bool QTorrentHandle::has_missing_files() const {
+  const QStringList paths = files_path();
+  foreach(const QString &path, paths) {
+    if(!QFile::exists(path)) return true;
+  }
+  return false;
+}
+
 int QTorrentHandle::queue_position() const {
   Q_ASSERT(h.is_valid());
   if(h.queue_position() < 0)
@@ -585,7 +593,16 @@ void QTorrentHandle::queue_position_up() const {
   Q_ASSERT(h.is_valid());
   if(h.queue_position() > 0)
     h.queue_position_up();
+}
 
+void QTorrentHandle::queue_position_top() const {
+  Q_ASSERT(h.is_valid());
+  h.queue_position_top();
+}
+
+void QTorrentHandle::queue_position_bottom() const {
+  Q_ASSERT(h.is_valid());
+    h.queue_position_bottom();
 }
 
 void QTorrentHandle::force_reannounce() {
