@@ -110,6 +110,7 @@ class QTorrentHandle {
     size_type all_time_download() const;
     size_type total_done() const;
     QStringList files_path() const;
+    bool has_missing_files() const;
     int num_uploads() const;
     bool is_seed() const;
     bool is_checking() const;
@@ -157,6 +158,8 @@ class QTorrentHandle {
     void set_tracker_login(QString username, QString password);
     void queue_position_down() const;
     void queue_position_up() const;
+    void queue_position_top() const;
+    void queue_position_bottom() const;
     void auto_managed(bool) const;
     void force_recheck() const;
     void move_storage(QString path) const;
