@@ -30,7 +30,7 @@
 
 #include "torrentadditiondlg.h"
 
-torrentAdditionDialog::torrentAdditionDialog(GUI *parent, Bittorrent* _BTSession) : QDialog((QWidget*)parent), old_label(""), hidden_height(0) {
+torrentAdditionDialog::torrentAdditionDialog(GUI *parent, Bittorrent* _BTSession) : QDialog((QWidget*)parent), old_label(""), hidden_height(0), cursor_pos(-1) {
   setupUi(this);
   setAttribute(Qt::WA_DeleteOnClose);
   connect(this, SIGNAL(torrentPaused(QTorrentHandle&)), parent->getTransferList(), SLOT(pauseTorrent(QTorrentHandle&)));
@@ -120,9 +120,30 @@ void torrentAdditionDialog::saveSettings() {
 }
 
 void torrentAdditionDialog::renameTorrentNameInModel(QString file_path) {
+  file_path = file_path.trimmed();
+  if(file_path.isEmpty()) return;
   file_path = file_path.replace("\\", "/");
   // Rename in torrent files model too
-  PropListModel->setData(PropListModel->index(0, 0), file_path.split("/", QString::SkipEmptyParts).last());
+  QStringList parts = file_path.split("/", QString::SkipEmptyParts);
+  if(!parts.empty())
+    PropListModel->setData(PropListModel->index(0, 0), parts.last());
+}
+
+void torrentAdditionDialog::limitDialogWidth() {
+  int scrn = 0;
+  const QWidget *w = this->window();
+
+  if(w)
+    scrn = QApplication::desktop()->screenNumber(w);
+  else if(QApplication::desktop()->isVirtualDesktop())
+    scrn = QApplication::desktop()->screenNumber(QCursor::pos());
+  else
+    scrn = QApplication::desktop()->screenNumber(this);
+
+  QRect desk(QApplication::desktop()->availableGeometry(scrn));
+  int max_width = desk.width();
+  if(max_width > 0)
+    setMaximumWidth(max_width);
 }
 
 void torrentAdditionDialog::hideTorrentContent() {
@@ -167,6 +188,8 @@ void torrentAdditionDialog::showLoadMagnetURI(QString magnet_uri) {
 
   // No need to display torrent content
   hideTorrentContent();
+  // Limit dialog width
+  limitDialogWidth();
 }
 
 void torrentAdditionDialog::showLoad(QString filePath, QString from_url) {
@@ -260,6 +283,9 @@ void torrentAdditionDialog::showLoad(QString filePath, QString from_url) {
   // Hide useless widgets
   if(t->num_files() <= 1)
     hideTorrentContent();
+
+  // Limit dialog width
+  limitDialogWidth();
 }
 
 void torrentAdditionDialog::displayContentListMenu(const QPoint&) {
@@ -313,8 +339,8 @@ void torrentAdditionDialog::renameSelectedFile() {
   // Ask for new name
   bool ok;
   const QString new_name_last = QInputDialog::getText(this, tr("Rename the file"),
-                                                       tr("New name:"), QLineEdit::Normal,
-                                                       index.data().toString(), &ok);
+                                                      tr("New name:"), QLineEdit::Normal,
+                                                      index.data().toString(), &ok);
   if (ok && !new_name_last.isEmpty()) {
     if(!misc::isValidFileSystemName(new_name_last)) {
       QMessageBox::warning(this, tr("The file could not be renamed"),
@@ -513,6 +539,7 @@ void torrentAdditionDialog::renameSelectedFile() {
       save_path = save_path.replace("\\", "/");
 #endif
       save_path = misc::expandPath(save_path);
+      qDebug("Save path is %s", qPrintable(save_path));
       if(!is_magnet && t->num_files() == 1) {
         // Remove file name
         QStringList parts = save_path.split("/");
@@ -521,6 +548,7 @@ void torrentAdditionDialog::renameSelectedFile() {
         files_path << single_file_name;
         save_path = parts.join("/");
       }
+      qDebug("Save path dir is %s", qPrintable(save_path));
       QDir savePath(save_path);
       const QString current_label = comboLabel->currentText().trimmed();
       if (!current_label.isEmpty() && !misc::isValidFileSystemName(current_label)) {
@@ -528,7 +556,8 @@ void torrentAdditionDialog::renameSelectedFile() {
         return;
       }
       // Save savepath
-      TorrentTempData::setSavePath(hash, savePath.path());
+      qDebug("Saving save path to temp data: %s", qPrintable(savePath.absolutePath()));
+      TorrentTempData::setSavePath(hash, savePath.absolutePath());
       qDebug("Torrent label is: %s", qPrintable(comboLabel->currentText().trimmed()));
       if(!current_label.isEmpty())
         TorrentTempData::setLabel(hash, current_label);
@@ -614,8 +643,18 @@ void torrentAdditionDialog::renameSelectedFile() {
         }
       }
 
+      void torrentAdditionDialog::restoreCursorPosition() {
+        savePathTxt->lineEdit()->setCursorPosition(cursor_pos);
+        cursor_pos = -1;
+      }
+
       void torrentAdditionDialog::updateSavePathCurrentText(QString path) {
+        if(cursor_pos >= 0)
+          return;
+        Q_UNUSED(path);
+        cursor_pos = savePathTxt->lineEdit()->cursorPosition();
         savePathTxt->setItemText(savePathTxt->currentIndex(), path);
+        QTimer::singleShot(0, this, SLOT(restoreCursorPosition()));
         path_history.replace(savePathTxt->currentIndex(), getCurrentTruncatedSavePath());
         QString root_folder_or_file_name = "";
         getCurrentTruncatedSavePath(&root_folder_or_file_name);
