@@ -38,6 +38,7 @@
 #include "GUI.h"
 #include "preferences.h"
 #include "deletionconfirmationdlg.h"
+#include "propertieswidget.h"
 #include <libtorrent/version.hpp>
 #include <QStandardItemModel>
 #include <QSortFilterProxyModel>
@@ -98,12 +99,18 @@ TransferListWidget::TransferListWidget(QWidget *parent, GUI *main_window, Bittor
   labelFilterModel->setFilterKeyColumn(TR_LABEL);
   labelFilterModel->setFilterRole(Qt::DisplayRole);
 
-  proxyModel = new QSortFilterProxyModel();
-  proxyModel->setDynamicSortFilter(true);
-  proxyModel->setSourceModel(labelFilterModel);
-  proxyModel->setFilterKeyColumn(TR_STATUS);
-  proxyModel->setFilterRole(Qt::DisplayRole);
-  setModel(proxyModel);
+  statusFilterModel = new QSortFilterProxyModel();
+  statusFilterModel->setDynamicSortFilter(true);
+  statusFilterModel->setSourceModel(labelFilterModel);
+  statusFilterModel->setFilterKeyColumn(TR_STATUS);
+  statusFilterModel->setFilterRole(Qt::DisplayRole);
+
+  nameFilterModel = new QSortFilterProxyModel();
+  nameFilterModel->setDynamicSortFilter(true);
+  nameFilterModel->setSourceModel(statusFilterModel);
+  nameFilterModel->setFilterKeyColumn(TR_NAME);
+  nameFilterModel->setFilterRole(Qt::DisplayRole);
+  setModel(nameFilterModel);
 
 
   // Visual settings
@@ -154,7 +161,8 @@ TransferListWidget::~TransferListWidget() {
   // Clean up
   delete refreshTimer;
   delete labelFilterModel;
-  delete proxyModel;
+  delete statusFilterModel;
+  delete nameFilterModel;
   delete listModel;
   delete listDelegate;
 }
@@ -209,7 +217,7 @@ void TransferListWidget::addTorrent(QTorrentHandle& h) {
     }
     // Select first torrent to be added
     if(listModel->rowCount() == 1)
-      selectionModel()->setCurrentIndex(proxyModel->index(row, TR_NAME), QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows);
+      selectionModel()->setCurrentIndex(nameFilterModel->index(row, TR_NAME), QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows);
     // Emit signal
     emit torrentAdded(listModel->index(row, 0));
     // Refresh the list
@@ -251,6 +259,7 @@ void TransferListWidget::pauseTorrent(int row, bool refresh_list) {
   const QTorrentHandle h = BTSession->getTorrentHandle(getHashFromRow(row));
   listModel->setData(listModel->index(row, TR_DLSPEED), QVariant((double)0.0));
   listModel->setData(listModel->index(row, TR_UPSPEED), QVariant((double)0.0));
+  listModel->setData(listModel->index(row, TR_PROGRESS), h.progress());
   if(h.is_seed()) {
     listModel->setData(listModel->index(row, TR_STATUS), STATE_PAUSED_UP);
     if(h.has_error() || TorrentPersistentData::hasError(h.hash())) {
@@ -303,6 +312,7 @@ void TransferListWidget::resumeTorrent(int row, bool refresh_list) {
 }
 
 void TransferListWidget::updateMetadata(QTorrentHandle &h) {
+  if(!h.is_valid()) return;
   const QString hash = h.hash();
   const int row = getRowFromHash(hash);
   if(row != -1) {
@@ -392,6 +402,8 @@ int TransferListWidget::updateTorrent(int row) {
     }
 
     if(h.is_paused()) {
+      // XXX: Force progress update because of bug #621381
+      listModel->setData(listModel->index(row, TR_PROGRESS), QVariant((double)h.progress()));
       if(h.is_seed())
         return STATE_PAUSED_UP;
       return STATE_PAUSED_DL;
@@ -572,11 +584,18 @@ inline QString TransferListWidget::getHashFromRow(int row) const {
 }
 
 inline QModelIndex TransferListWidget::mapToSource(const QModelIndex &index) const {
-  return labelFilterModel->mapToSource(proxyModel->mapToSource(index));
+  Q_ASSERT(index.isValid());
+  if(index.model() == nameFilterModel)
+    return labelFilterModel->mapToSource(statusFilterModel->mapToSource(nameFilterModel->mapToSource(index)));
+  if(index.model() == statusFilterModel)
+    return labelFilterModel->mapToSource(statusFilterModel->mapToSource(index));
+  return labelFilterModel->mapToSource(index);
 }
 
 inline QModelIndex TransferListWidget::mapFromSource(const QModelIndex &index) const {
-  return proxyModel->mapFromSource(labelFilterModel->mapFromSource(index));
+  Q_ASSERT(index.isValid());
+  Q_ASSERT(index.model() == labelFilterModel);
+  return nameFilterModel->mapFromSource(statusFilterModel->mapFromSource(labelFilterModel->mapFromSource(index)));
 }
 
 
@@ -630,7 +649,8 @@ void TransferListWidget::setSelectedTorrentsLocation() {
   const QStringList hashes = getSelectedTorrentsHashes();
   if(hashes.isEmpty()) return;
   QString dir;
-  const QDir saveDir(BTSession->getTorrentHandle(hashes.first()).save_path());
+  const QDir saveDir(TorrentPersistentData::getSavePath(hashes.first()));
+  qDebug("Torrent save path is %s", qPrintable(saveDir.absolutePath()));
   if(saveDir.exists()){
     dir = QFileDialog::getExistingDirectory(this, tr("Choose save path"), saveDir.path());
   }else{
@@ -648,8 +668,12 @@ void TransferListWidget::setSelectedTorrentsLocation() {
     foreach(const QString & hash, hashes) {
       // Actually move storage
       QTorrentHandle h = BTSession->getTorrentHandle(hash);
-      if(!BTSession->useTemporaryFolder() || h.is_seed())
+      if(!BTSession->useTemporaryFolder() || h.is_seed()) {
         h.move_storage(savePath.absolutePath());
+      } else {
+        TorrentPersistentData::saveSavePath(h.hash(), savePath.absolutePath());
+        main_window->getProperties()->updateSavePath(h);
+      }
     }
   }
 }
@@ -678,6 +702,22 @@ void TransferListWidget::startAllTorrents() {
   refreshList();
 }
 
+void TransferListWidget::startVisibleTorrents() {
+  QStringList hashes;
+  for(int i=0; i<nameFilterModel->rowCount(); ++i) {
+    const int row = mapToSource(nameFilterModel->index(i, 0)).row();
+    hashes << getHashFromRow(row);
+  }
+  foreach(const QString &hash, hashes) {
+    QTorrentHandle h = BTSession->getTorrentHandle(hash);
+    if(h.is_valid() && h.is_paused()) {
+      h.resume();
+      resumeTorrent(getRowFromHash(hash), false);
+    }
+  }
+  refreshList();
+}
+
 void TransferListWidget::pauseSelectedTorrents() {
   const QStringList hashes = getSelectedTorrentsHashes();
   foreach(const QString &hash, hashes) {
@@ -702,6 +742,22 @@ void TransferListWidget::pauseAllTorrents() {
   refreshList();
 }
 
+void TransferListWidget::pauseVisibleTorrents() {
+  QStringList hashes;
+  for(int i=0; i<nameFilterModel->rowCount(); ++i) {
+    const int row = mapToSource(nameFilterModel->index(i, 0)).row();
+    hashes << getHashFromRow(row);
+  }
+  foreach(const QString &hash, hashes) {
+    QTorrentHandle h = BTSession->getTorrentHandle(hash);
+    if(h.is_valid() && !h.is_paused()) {
+      h.pause();
+      pauseTorrent(getRowFromHash(hash), false);
+    }
+  }
+  refreshList();
+}
+
 void TransferListWidget::deleteSelectedTorrents() {
   if(main_window->getCurrentTabWidget() != this) return;
   const QStringList& hashes = getSelectedTorrentsHashes();
@@ -718,8 +774,26 @@ void TransferListWidget::deleteSelectedTorrents() {
   }
 }
 
-// FIXME: Should work only if the tab is displayed
+void TransferListWidget::deleteVisibleTorrents() {
+  if(nameFilterModel->rowCount() <= 0) return;
+  bool delete_local_files = false;
+  if(DeletionConfirmationDlg::askForDeletionConfirmation(&delete_local_files)) {
+    QStringList hashes;
+    for(int i=0; i<nameFilterModel->rowCount(); ++i) {
+      const int row = mapToSource(nameFilterModel->index(i, 0)).row();
+      hashes << getHashFromRow(row);
+    }
+    foreach(const QString &hash, hashes) {
+      const int row = getRowFromHash(hash);
+      deleteTorrent(row, false);
+      BTSession->deleteTorrent(hash, delete_local_files);
+    }
+    refreshList();
+  }
+}
+
 void TransferListWidget::increasePrioSelectedTorrents() {
+  if(main_window->getCurrentTabWidget() != this) return;
   const QStringList hashes = getSelectedTorrentsHashes();
   foreach(const QString &hash, hashes) {
     QTorrentHandle h = BTSession->getTorrentHandle(hash);
@@ -730,8 +804,8 @@ void TransferListWidget::increasePrioSelectedTorrents() {
   refreshList();
 }
 
-// FIXME: Should work only if the tab is displayed
 void TransferListWidget::decreasePrioSelectedTorrents() {
+  if(main_window->getCurrentTabWidget() != this) return;
   const QStringList hashes = getSelectedTorrentsHashes();
   foreach(const QString &hash, hashes) {
     QTorrentHandle h = BTSession->getTorrentHandle(hash);
@@ -742,6 +816,30 @@ void TransferListWidget::decreasePrioSelectedTorrents() {
   refreshList();
 }
 
+void TransferListWidget::topPrioSelectedTorrents() {
+  if(main_window->getCurrentTabWidget() != this) return;
+  const QStringList hashes = getSelectedTorrentsHashes();
+  foreach(const QString &hash, hashes) {
+    QTorrentHandle h = BTSession->getTorrentHandle(hash);
+    if(h.is_valid() && !h.is_seed()) {
+      h.queue_position_top();
+    }
+  }
+  refreshList();
+}
+
+void TransferListWidget::bottomPrioSelectedTorrents() {
+  if(main_window->getCurrentTabWidget() != this) return;
+  const QStringList hashes = getSelectedTorrentsHashes();
+  foreach(const QString &hash, hashes) {
+    QTorrentHandle h = BTSession->getTorrentHandle(hash);
+    if(h.is_valid() && !h.is_seed()) {
+      h.queue_position_bottom();
+    }
+  }
+  refreshList();
+}
+
 void TransferListWidget::buySelectedTorrents() const {
   const QStringList hashes = getSelectedTorrentsHashes();
   foreach(const QString &hash, hashes) {
@@ -1002,7 +1100,7 @@ void TransferListWidget::renameSelectedTorrent() {
     // Remember the name
     TorrentPersistentData::saveName(hash, name);
     // Visually change the name
-    proxyModel->setData(selectedIndexes.first(), name);
+    nameFilterModel->setData(selectedIndexes.first(), name);
   }
 }
 
@@ -1037,11 +1135,11 @@ void TransferListWidget::removeLabelFromRows(QString label) {
 
 void TransferListWidget::displayListMenu(const QPoint&) {
   // Create actions
-  QAction actionStart(QIcon(QString::fromUtf8(":/Icons/skin/play.png")), tr("Start"), 0);
+  QAction actionStart(QIcon(QString::fromUtf8(":/Icons/skin/play.png")), tr("Resume", "Resume/start the torrent"), 0);
   connect(&actionStart, SIGNAL(triggered()), this, SLOT(startSelectedTorrents()));
-  QAction actionPause(QIcon(QString::fromUtf8(":/Icons/skin/pause.png")), tr("Pause"), 0);
+  QAction actionPause(QIcon(QString::fromUtf8(":/Icons/skin/pause.png")), tr("Pause", "Pause the torrent"), 0);
   connect(&actionPause, SIGNAL(triggered()), this, SLOT(pauseSelectedTorrents()));
-  QAction actionDelete(QIcon(QString::fromUtf8(":/Icons/skin/delete.png")), tr("Delete"), 0);
+  QAction actionDelete(QIcon(QString::fromUtf8(":/Icons/skin/delete.png")), tr("Delete", "Delete the torrent"), 0);
   connect(&actionDelete, SIGNAL(triggered()), this, SLOT(deleteSelectedTorrents()));
   QAction actionPreview_file(QIcon(QString::fromUtf8(":/Icons/skin/preview.png")), tr("Preview file..."), 0);
   connect(&actionPreview_file, SIGNAL(triggered()), this, SLOT(previewSelectedTorrents()));
@@ -1053,12 +1151,16 @@ void TransferListWidget::displayListMenu(const QPoint&) {
   connect(&actionOpen_destination_folder, SIGNAL(triggered()), this, SLOT(openSelectedTorrentsFolder()));
   //QAction actionBuy_it(QIcon(QString::fromUtf8(":/Icons/oxygen/wallet.png")), tr("Buy it"), 0);
   //connect(&actionBuy_it, SIGNAL(triggered()), this, SLOT(buySelectedTorrents()));
+  QAction actionIncreasePriority(QIcon(QString::fromUtf8(":/Icons/oxygen/go-up.png")), tr("Move up", "i.e. move up in the queue"), 0);
+  connect(&actionIncreasePriority, SIGNAL(triggered()), this, SLOT(increasePrioSelectedTorrents()));
+  QAction actionDecreasePriority(QIcon(QString::fromUtf8(":/Icons/oxygen/go-down.png")), tr("Move down", "i.e. Move down in the queue"), 0);
+  connect(&actionDecreasePriority, SIGNAL(triggered()), this, SLOT(decreasePrioSelectedTorrents()));
+  QAction actionTopPriority(QIcon(QString::fromUtf8(":/Icons/oxygen/go-top.png")), tr("Move to top", "i.e. Move to top of the queue"), 0);
+  connect(&actionTopPriority, SIGNAL(triggered()), this, SLOT(topPrioSelectedTorrents()));
+  QAction actionBottomPriority(QIcon(QString::fromUtf8(":/Icons/oxygen/go-bottom.png")), tr("Move to bottom", "i.e. Move to bottom of the queue"), 0);
+  connect(&actionBottomPriority, SIGNAL(triggered()), this, SLOT(bottomPrioSelectedTorrents()));
   QAction actionSetTorrentPath(QIcon(QString::fromUtf8(":/Icons/skin/folder.png")), tr("Set location..."), 0);
   connect(&actionSetTorrentPath, SIGNAL(triggered()), this, SLOT(setSelectedTorrentsLocation()));
-  QAction actionIncreasePriority(QIcon(QString::fromUtf8(":/Icons/skin/increase.png")), tr("Increase priority"), 0);
-  connect(&actionIncreasePriority, SIGNAL(triggered()), this, SLOT(increasePrioSelectedTorrents()));
-  QAction actionDecreasePriority(QIcon(QString::fromUtf8(":/Icons/skin/decrease.png")), tr("Decrease priority"), 0);
-  connect(&actionDecreasePriority, SIGNAL(triggered()), this, SLOT(decreasePrioSelectedTorrents()));
   QAction actionForce_recheck(QIcon(QString::fromUtf8(":/Icons/oxygen/gear.png")), tr("Force recheck"), 0);
   connect(&actionForce_recheck, SIGNAL(triggered()), this, SLOT(recheckSelectedTorrents()));
   QAction actionCopy_magnet_link(QIcon(QString::fromUtf8(":/Icons/magnet.png")), tr("Copy magnet link"), 0);
@@ -1199,8 +1301,11 @@ void TransferListWidget::displayListMenu(const QPoint&) {
   listMenu.addAction(&actionOpen_destination_folder);
   if(BTSession->isQueueingEnabled() && one_not_seed) {
     listMenu.addSeparator();
-    listMenu.addAction(&actionIncreasePriority);
-    listMenu.addAction(&actionDecreasePriority);
+    QMenu *prioMenu = listMenu.addMenu(tr("Priority"));
+    prioMenu->addAction(&actionTopPriority);
+    prioMenu->addAction(&actionIncreasePriority);
+    prioMenu->addAction(&actionDecreasePriority);
+    prioMenu->addAction(&actionBottomPriority);
   }
   listMenu.addSeparator();
   if(one_has_metadata)
@@ -1328,6 +1433,7 @@ void TransferListWidget::loadLastSortedColumn() {
 }
 
 void TransferListWidget::currentChanged(const QModelIndex& current, const QModelIndex&) {
+  qDebug("CURRENT CHANGED");
   QTorrentHandle h;
   if(current.isValid()) {
     const int row = mapToSource(current).row();
@@ -1351,32 +1457,36 @@ void TransferListWidget::applyLabelFilter(QString label) {
   labelFilterModel->setFilterRegExp(QRegExp("^"+label+"$", Qt::CaseSensitive));
 }
 
+void TransferListWidget::applyNameFilter(QString name) {
+  nameFilterModel->setFilterRegExp(QRegExp(name, Qt::CaseInsensitive));
+}
+
 void TransferListWidget::applyStatusFilter(int f) {
   switch(f) {
   case FILTER_DOWNLOADING:
-    proxyModel->setFilterRegExp(QRegExp(QString::number(STATE_DOWNLOADING)+"|"+QString::number(STATE_STALLED_DL)+"|"+
+    statusFilterModel->setFilterRegExp(QRegExp(QString::number(STATE_DOWNLOADING)+"|"+QString::number(STATE_STALLED_DL)+"|"+
                                         QString::number(STATE_PAUSED_DL)+"|"+QString::number(STATE_CHECKING_DL)+"|"+
                                         QString::number(STATE_QUEUED_DL), Qt::CaseSensitive));
     break;
   case FILTER_COMPLETED:
-    proxyModel->setFilterRegExp(QRegExp(QString::number(STATE_SEEDING)+"|"+QString::number(STATE_STALLED_UP)+"|"+
+    statusFilterModel->setFilterRegExp(QRegExp(QString::number(STATE_SEEDING)+"|"+QString::number(STATE_STALLED_UP)+"|"+
                                         QString::number(STATE_PAUSED_UP)+"|"+QString::number(STATE_CHECKING_UP)+"|"+
                                         QString::number(STATE_QUEUED_UP), Qt::CaseSensitive));
     break;
   case FILTER_ACTIVE:
-    proxyModel->setFilterRegExp(QRegExp(QString::number(STATE_DOWNLOADING)+"|"+QString::number(STATE_SEEDING), Qt::CaseSensitive));
+    statusFilterModel->setFilterRegExp(QRegExp(QString::number(STATE_DOWNLOADING)+"|"+QString::number(STATE_SEEDING), Qt::CaseSensitive));
     break;
   case FILTER_INACTIVE:
-    proxyModel->setFilterRegExp(QRegExp("[^"+QString::number(STATE_DOWNLOADING)+QString::number(STATE_SEEDING)+"]", Qt::CaseSensitive));
+    statusFilterModel->setFilterRegExp(QRegExp("[^"+QString::number(STATE_DOWNLOADING)+QString::number(STATE_SEEDING)+"]", Qt::CaseSensitive));
     break;
   case FILTER_PAUSED:
-    proxyModel->setFilterRegExp(QRegExp(QString::number(STATE_PAUSED_UP)+"|"+QString::number(STATE_PAUSED_DL)));
+    statusFilterModel->setFilterRegExp(QRegExp(QString::number(STATE_PAUSED_UP)+"|"+QString::number(STATE_PAUSED_DL)));
     break;
   default:
-    proxyModel->setFilterRegExp(QRegExp());
+    statusFilterModel->setFilterRegExp(QRegExp());
   }
   // Select first item if nothing is selected
-  if(selectionModel()->selectedRows(0).empty() && proxyModel->rowCount() > 0)
-    selectionModel()->setCurrentIndex(proxyModel->index(0, TR_NAME), QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows);
+  if(selectionModel()->selectedRows(0).empty() && statusFilterModel->rowCount() > 0)
+    selectionModel()->setCurrentIndex(statusFilterModel->index(0, TR_NAME), QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows);
 }
 
