@@ -63,11 +63,16 @@ public slots:
   void setSelectedTorrentsLocation();
   void startSelectedTorrents();
   void startAllTorrents();
+  void startVisibleTorrents();
   void pauseSelectedTorrents();
   void pauseAllTorrents();
+  void pauseVisibleTorrents();
   void deleteSelectedTorrents();
+  void deleteVisibleTorrents();
   void increasePrioSelectedTorrents();
   void decreasePrioSelectedTorrents();
+  void topPrioSelectedTorrents();
+  void bottomPrioSelectedTorrents();
   void buySelectedTorrents() const;
   void copySelectedMagnetURIs() const;
   void openSelectedTorrentsFolder() const;
@@ -77,6 +82,7 @@ public slots:
   void previewSelectedTorrents();
   void hidePriorityColumn(bool hide);
   void displayDLHoSMenu(const QPoint&);
+  void applyNameFilter(QString name);
   void applyStatusFilter(int f);
   void applyLabelFilter(QString label);
   void previewFile(QString filePath);
@@ -125,7 +131,8 @@ signals:
 private:
   TransferListDelegate *listDelegate;
   QStandardItemModel *listModel;
-  QSortFilterProxyModel *proxyModel;
+  QSortFilterProxyModel *nameFilterModel;
+  QSortFilterProxyModel *statusFilterModel;
   QSortFilterProxyModel *labelFilterModel;
   Bittorrent* BTSession;
   QTimer *refreshTimer;
